import torch
import numpy as np

def loadnpy(f):
    return np.load(f, allow_pickle=True).tolist()

def getSrc(src, tgt, prediction, tenor, vehicle):
    t_out = ""
    p_out = ""
    for w in src:
        if w == "[MASK]":
            t_out += " ("+tgt+") "
            p_out += " ("+prediction+") "
        elif w == "[PAD]":
            break
        else:
            t_out += w
            p_out += w
    t_out = t_out.replace(",", "，")
    p_out = p_out.replace(",", "，")

    t_out = ", ".join([t_out, tenor, vehicle])
    p_out = ", ".join([p_out, tenor, vehicle])
    return [t_out, p_out]

def getString(src, tgt):
    src_str = ""
    for w in src:
        if w == "[MASK]":
            src_str += tgt
        elif w == "[PAD]":
            break
        else:
            src_str += w
    src_str = src_str.replace(",", "，")
    return src_str

def main():
    data_origin = loadnpy("./data/snapshot/data_origin.npy")
    data_eval = loadnpy("./data/snapshot/data_eval.npy")

    rhyme_predictions = loadnpy("./data/snapshot/rhyme_predictions.npy")
    eval_rhyme_idxs = loadnpy("./data/snapshot/eval_rhyme_idxs.npy")

    glyph_candidates = loadnpy("./data/snapshot/glyph_candidates.npy")

    eval_len = 10750

    # normal_f = open("./data/expert_eval/normal.csv", "w")
    # normal_idx_path = "./data/expert_eval/normal_idx.npy"
    # normal_label_path = "./data/expert_eval/normal_label.npy"

    rhyme_f = open("./data/expert_eval/rhyme.csv", "w")
    rhyme_idx_path = "./data/expert_eval/rhyme_idx.npy"
    rhyme_label_path = "./data/expert_eval/rhyme_label.npy"

    # # 280 normal prediction
    # while True:
    #     normal_idx = np.random.randint(0, eval_len, 280)
    #     if len(set(normal_idx)) == 280:
    #         break
    # normal_label = np.random.randint(0, 2, 280)
    # for i, idx in enumerate(normal_idx):
    #     data = data_origin[data_eval[idx]["id"]]
    #     [t_out, p_out] = getSrc(data["src"], data["tgt"], glyph_candidates[idx][0], data["tenor"], data["vehicle"])
    #     normal_f.write("-\n")
    #     if normal_label[i] == 0:
    #         normal_f.write(t_out+"\n")
    #         normal_f.write(p_out+"\n")
    #     else:
    #         normal_f.write(p_out+"\n")
    #         normal_f.write(t_out+"\n")
    # np.save(normal_idx_path, normal_idx)
    # np.save(normal_label_path, normal_label)

    # 10 rhyme prediction
    # while True:
    #     rhyme_idx = np.random.randint(0, 28, 20)
    #     if len(set(rhyme_idx)) == 20:
    #         break
    # rhyme_label = np.random.randint(0, 2, 20)
    rhyme_idx = np.arange(28)
    rhyme_label =np.random.randint(0, 2, 28)

    for i, idx in enumerate(rhyme_idx):
        data_idx = data_eval[eval_rhyme_idxs[idx]]["id"]
        data = data_origin[data_idx]
        [t_out, p_out] = getSrc(data["src"], data["tgt"], rhyme_predictions[idx]["w"], data["tenor"], data["vehicle"])
        data_origin_line_idx = data_eval[eval_rhyme_idxs[idx]]["line_idx"]
        [start, end] = rhyme_predictions[idx]["span"]
        t_outs = []
        p_outs = []
        for j in range(data_idx-data_origin_line_idx+start, data_idx-data_origin_line_idx+end+1):
            if j == data_idx:
                t_outs.append(t_out+", "+rhyme_predictions[idx]["des"])
                p_outs.append(p_out+", "+rhyme_predictions[idx]["des"])
            else:
                t_outs.append(getString(data_origin[j]["src"], data_origin[j]["tgt"]))
                p_outs.append(getString(data_origin[j]["src"], data_origin[j]["tgt"]))
        rhyme_f.write("-\n")
        for s in t_outs:
            rhyme_f.write(s+"\n")
        rhyme_f.write("\n")
        for s in p_outs:
            rhyme_f.write(s+"\n")
    np.save(rhyme_idx_path, rhyme_idx)
    np.save(rhyme_label_path, rhyme_label)

if __name__ == "__main__":
    main()
