from pypinyin import pinyin, Style
from settings import *

class Util(object):
    """docstring for Util"""
    def __init__(self, arg):
        super(Util, self).__init__()
        self.arg = arg
        
    # - Check if the pair of pinyin is rhyme or not
    def isRhyme(p_i1, p_f_1, p_i2, p_f_2):    # p_i:initials
                                                # p_f:finals
        def pinyin2idx(p_f):
            for idx in range(len(RHYMES)):
                if p_f in RHYMES[idx]:
                    return idx
            return False

        if pinyin2idx(p_i1, p_f_1)==pinyin2idx(p_i2, p_f_2):
            return True
        else:
            return False

    # - Convert Chinese character to finals
    def c2final(c):
        def isSP1(p_i, p_f):    # -i, like yi
                                # in pypinyin, initial of "椅" is ""
                                # but initial of "只" is "zh"
            if p_f==RHYMES_SP1[0] and len(p_i)<2:
                return True
            else:
                return False
        def isSP2(p_i, p_f):    # --i, like zhi
            if len(p_i)==2 and p_f==RHYMES_SP2[0]:
                return True
            else:
                return False
        def isSP3(p_i, p_f):    # ye
            if (p_i+p_f)==RHYMES_SP3[0]:
                return True
            else:
                return False

        p_i = pinyin(c, style=Style.INITIALS)[-1][-1]
        p_f = pinyin(c, style=Style.FINALS)[-1][-1]

        # pypinyin has some uncorrect convert, here to fix it
        if p_i == "":
            if p_f[0] == "i":
                p_i = "y"
                if len(p_f)>1 and p_f not in ["ing", "in"]:
                    p_f = p_f[1:]
            elif p_f[0] == "u":
                p_i = "w"
                if len(p_f)>1:
                    p_f = p_f[1:]
            elif p_f[0] == "n":
                p_i = ""
                p_f = "en"
            elif p_f[0] == "v":
                p_i = "y"
                p_f = p_f
        elif p_f in ["uei", "iou", "uen", "io", "ueng", "n"]:
            p_f = "ui" if p_f=="uei" else p_f
            p_f = "iu" if p_f=="iou" else p_f
            p_f = "un" if p_f=="uen" else p_f
            p_f = "o" if p_f=="io" else p_f
            p_f = "eng" if p_f=="ueng" else p_f

        if isSP1(p_i, p_f):
            p_f = "-i"
        elif isSP2(p_i, p_f):
            p_f = "--i"
        elif isSP3(p_i, p_f):
            p_f = "ye"

        if p_f not in FINALS:
            print(c)

        return p_f

    # - Produce N identical layers
    def clones(module, N):
        return nn.ModuleList([copy.deepcopy(module) for _ in range(N)])

    # - Mask out subsequent positions
    def subsequentMask(size):
        attn_shape = (1, size, size)
        subsequent_mask = np.triu(np.ones(attn_shape), k=1).astype('uint8')
        return torch.from_numpy(subsequent_mask) == 0

    # - Print progress bar
    def printProgressBar(iteration, total, prefix = '', suffix = '', \
            decimals = 5, length = 100, fill = '█', time_consumed
         = '', complete_message="Progress Done"):
        """
        Call in a loop to create terminal progress bar
        @params:
            iteration   - Required  : current iteration (Int)
            total       - Required  : total iterations (Int)
            prefix      - Optional  : prefix string (Str)
            suffix      - Optional  : suffix string (Str)
            decimals    - Optional  : positive number of decimals in percent complete (Int)
            length      - Optional  : character length of bar (Int)
            fill        - Optional  : bar fill character (Str)
        # # 
        # # Sample Usage
        # # 

        # from time import sleep

        # # A List of Items
        # items = list(range(0, 57))
        # l = len(items)

        # # Initial call to print 0% progress
        # printProgressBar(0, l, prefix = 'Progress:', suffix = 'Complete', length = 50)
        # for i, item in enumerate(items):
        #     # Do stuff...
        #     sleep(0.1)
        #     # Update Progress Bar
        #     printProgressBar(i + 1, l, prefix = 'Progress:', suffix = 'Complete', length = 50)
        """
        percent = 100 * (iteration / float(total))
        percent = ("{0:." + str(decimals) + "f}").format(min(percent, 100))
        filledLength = int(length * iteration // total)
        bar = fill * filledLength + '-' * (length - filledLength)
        print('\r%s |%s| %s%% %s %s' % (prefix, bar, percent, time_consumed, suffix), end = '\r')
        # Print New Line on Complete
        if iteration == total: 
            print(complete_message)

    